#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "hdfi.h"
#include "cris.h"
#include "FileTypes.h"
#include "crisPutData.h"
#include "utilities.h"
#include "crisSisPutUtilities.h"

/*
version 1.2:
	instead of merely appending new struct, create a new file
	placing the struct in the location which keeps the structs
	in order, time-wise.

version 1.1:
	change the storage of this data to be sequentially appended
	to file instead of placed in an interval.  this data is
	not coming in in a continuous stream as the others are.

version 1.0:
	initial version for putCrisCommandEcho.

putCrisCommandEcho is passed a pointer to a struct L1CrisCommandEcho
which it then attempts to archive inside a daily file in the cris data
archive directory tree in this manner:

a base data directory is gotten from the environment variable
L1_CRIS_DATA_BASE_DIRECTORY.  if this environment variable has
not been defined, putCrisCommandEcho exits.  for following examples,
let's say this has been defined as /ace/cris/data.  if /ace/cris/data
does not exist, putCrisCommandEcho exits.

putCrisCommandEcho reads the space craft seconds since 1996 and determines
the year, month, and day.  for example's sake, say year = 1999,
month = 5 (1-12), and day = 23 (1-31).

putCrisCommandEcho then checks for the existence of a directory named
/ace/cris/data/S1999.05/, and if not finding it, creates it.  the
same for the next directory level, /ace/cris/data/S1999.05/S1999.05.23/

putCrisCommandEcho then checks for the existence of the file
/ace/cris/data/S1999.05/S1999.05.23/S1999.05.23.ce
if this file is not found, it is created and initialized with
an 8 word header. 

putCrisCommandEcho then reads what is already in this file.  if the
data struct is identical to another already in the file, nothing
is written to file, otherwise the data is inserted into the file
in time ordered sequence.  the return value in the case where an identical
data structure is found is 3.  otherwise putCrisCommandEcho
returns a 0, signalling the struct has been appended to the file
normally.

a return value of 1 was added in version 1.2 to signal that a
non-identical struct having the same time stamp as the passed
struct was found in the file.  in this case, the original file
is left unchanged, and the passed struct is written to an
overlap file (signified as same file name, appended by ".overlap").

*/

int putCrisCommandEcho(struct L1CrisCommandEcho *pDataStruct) {
	/** struct type dependent code **/
    unsigned int DATA_TYPE_ID = FILE_CRIS_COMMAND_ECHO;
    char FILE_TYPE_SUFFIX[] = "ce";

	/* declared static so memory allocation occurs only once */
	/** struct type dependent code **/
    static char dataBuffer[sizeof(struct L1CrisCommandEcho)];
    char *pDataBuffer, *pPassedData;

    int fd, tempFd, i, intervalNumber, bytesRead;
    int foundIdenticalDataStructInFile;
    char path[256], tempPath[256];
    struct stat oldFileStats, newFileStats;

	/* intervalNumber is unused in putCrisCommandEcho */
	/** struct type dependent code **/
    determineDataFilePathAndCreateDirsAsNeeded(pDataStruct->Second1996,
				FILE_TYPE_SUFFIX, path, &intervalNumber, 'C');
    /* fprintf(stderr, "path = %s, intervalNumber = %d\n", path,
						pDataStruct->Second1996); */

    pDataBuffer = dataBuffer;
    if ((fd = open(path, O_RDWR)) == -1) {	/* file doesn't exist yet? */
	/* fprintf(stderr, "file %s does not exist\n", path); */
	fd = openOrCreateFileToReadWrite(path);
	writeHeader(fd, DATA_TYPE_ID);
	    /* since this is a new file, just write the struct and return */
	writeToFile(fd, (char *) pDataStruct, sizeof(dataBuffer));
	close(fd);
	return 0;
    }

	/* file already exists.  first read the last struct in the file */
	/* and see if we just need to append the passed struct to file's end */

	/* go to end of file, and back up one struct size */
    lseek(fd, -1 * sizeof(dataBuffer), 2);
    readFromFile(fd, dataBuffer, sizeof(dataBuffer));
	/* is new struct's time stamp past last struct in file's time stamp? */
	/** struct type dependent code **/
    if (pDataStruct->Second1996 >
			((struct L1CrisCommandEcho *) dataBuffer)->Second1996) {
	    /* append passed struct and return */
	lseek(fd, 0L, 2);   /* go to end of file */
	writeToFile(fd, (char *) pDataStruct, sizeof(dataBuffer));
	close(fd);
	return 0;
    }

	/* the passed struct needs to be inserted into place, so we need */
	/* to rewrite the file and stick it in where it belongs... */

	/* create new temp file */
    strcpy(tempPath, path);
    strcat(tempPath, ".tmp");
    tempFd = openOrCreateFileToReadWrite(tempPath);
    writeHeader(tempFd, DATA_TYPE_ID);

	/* read from original file */
    lseek(fd, HEADER_BYTES, 0);		/* go to start of file, past header */
	/* read in the next data structure from file */
    while ((bytesRead = read(fd, dataBuffer, sizeof(dataBuffer)))) {
	/** struct type dependent code **/
	if (pDataStruct->Second1996 >
			((struct L1CrisCommandEcho *) dataBuffer)->Second1996) {
		/* struct in file is earlier than passed struct, write it out */
	    writeToFile(tempFd, dataBuffer, sizeof(dataBuffer));
	}
	else if (pDataStruct->Second1996 <
			((struct L1CrisCommandEcho *) dataBuffer)->Second1996) {
		/* found where passed struct needs to be inserted.  write it */
	    writeToFile(tempFd, (char *) pDataStruct, sizeof(dataBuffer));
		/* now write struct it was compared to just now */
	    writeToFile(tempFd, dataBuffer, sizeof(dataBuffer));
		/* get out of comparison loop */
	    break;
	}
	else {
		/* time stamps are identical.  no write to original file, */
		/* but write passed struct to overlap file if it is */
		/* different than the struct in the file.  if the two */
		/* structs are identical, just clean up and return. */
	    close(fd);
	    close(tempFd);
	    if (remove(tempPath)) {
		fprintf(stderr, "Couldn't remove %s.\n\nExiting.\n\n",tempPath);		exit(0);
	    }

	    pPassedData = (char *) pDataStruct;
	    pDataBuffer = dataBuffer;
	    for (i = 0; i < sizeof(dataBuffer); i++) {
		if (*pPassedData == *pDataBuffer) {
			/* this comparison needs to be true over all chars */
		    foundIdenticalDataStructInFile = 1;
		    pPassedData++;
		    pDataBuffer++;
		}
		else {
			/* this only needs to happen once in a struct */
		    foundIdenticalDataStructInFile = 0;
		    break;
		}
	    }
	    if (foundIdenticalDataStructInFile) {
		return 3;
	    }
	    else {
		    /* append passed struct to an overlap file and return. */
		strcpy(tempPath, path);
		strcat(tempPath, ".overlap");
		tempFd = openOrCreateFileToReadWrite(tempPath);
		lseek(tempFd, 0L, 2);	/* go to end of file */
		writeToFile(tempFd, (char *) pDataStruct, sizeof(dataBuffer));
		close(tempFd);
		return 1;
	    }
	}
    }
	/* now write out the rest of the structs in the original file */
    while ((bytesRead = read(fd, dataBuffer, sizeof(dataBuffer))))
			writeToFile(tempFd, dataBuffer, sizeof(dataBuffer));
    close(fd);
    close(tempFd);

	/* do a simple error check to make sure new file is larger than */
	/* the original file at least... */
    stat(path, &oldFileStats);
    stat(tempPath, &newFileStats);
    if (oldFileStats.st_size >= newFileStats.st_size) {
	fprintf(stderr, "New file %s is not larger than old file %s.\n\n",
								tempPath, path);
	fprintf(stderr, "Exiting.\n\n");
	exit(0);
    }
    else {
	    /* finally, replace the old file with the new file */
	if (remove(path)) {
	    fprintf(stderr, "Couldn't remove %s.\n\nExiting.\n\n", path);
	    exit(0);
	}

	if (rename(tempPath, path)) {
	    fprintf(stderr, "Couldn't rename %s to %s.\n\nExiting.\n\n",
								tempPath, path);
	    exit(0);
	}
    }
    return 0;
}
