#include <stdio.h>
#include <fcntl.h>
#include <time.h>
#include "hdfi.h"
#include "cris.h"
#include "FileTypes.h"
#include "crisPutData.h"
#include "crisSisPutUtilities.h"
#include "utilities.h"

/*

version 2:
	initial version for putCrisSummary.

putCrisSummary is passed a pointer to a struct L1CrisSummary
which it then attempts to archive inside a daily file in the cris data
archive directory tree in this manner:

a base data directory is gotten from the environment variable
L1_CRIS_DATA_BASE_DIRECTORY.  if this environment variable has
not been defined, putCrisSummary exits.  for following examples,
let's say this has been defined as /ace/cris/data.  if /ace/cris/data
does not exist, putCrisSummary exits.

putCrisSummary reads the space craft seconds since 1996 and determines
the year, month, day, and seconds since midnight into the day.  for
example's sake, say year = 1999, month = 5 (1-12), day = 23 (1-31),
and seconds since midnight = 22174 (0-86399).

putCrisSummary then checks for the existence of a directory named
/ace/cris/data/C1999.05/, and if not finding it, creates it.  the
same for the next directory level, /ace/cris/data/C1999.05/C1999.05.23/

putCrisSummary then checks for the existence of the file
/ace/cris/data/C1999.05/C1999.05.23/C1999.05.23.sum
if this file is not found, it is created and initialized with
an 8 word header and the rest is zeroed.  the size of this file
is constant.  there is room in it for storing the 338 blocks
of 256 second intervals that make up one day (includes an extra
slot for the last fraction of an interval that occurs in a day.)

the 256 second block of data in the L1CrisSummary struct is then
placed into the day's data file in the location determined by
seconds-since-midnight / 256 (fractions are ignored), e.g.,
interval = 22174 / 256 = 86

this gives us a location in the file to place the L1CrisSummary struct:
location for file = (header size) + interval * (L1CrisSummary struct
size)

putCrisSummary then reads what is already at this location.  if it
is zeroed (from initialization), putCrisSummary writes the passed
L1CrisSummary struct into the file and returns a 0.  done.

however, there exists the possibility that, due to some quirk, 2
(or more) passed L1CrisSummary structs will lay claim to the same
location.  if this occurs, both the passed L1CrisSummary struct
and the L1CrisSummary struct already in the file are appended to
an auxiliary file called
/ace/cris/data/C1999.05/C1999.05.23/C1999.05.23.sum.overlap
the contested location in 
/ace/cris/data/C1999.05/C1999.05.23/C1999.05.23.sum
is then filled with char -1s to signal a reader to check the .overlap
file for the multiple structs.  putCrisSummary returns a 1 when
for the first contesting of a particular block and a 2 for every
susequent attempt to write into that particular position.  a 1 is
a signal that something weird happened.  a 2 means something very
strange is going on.

if it is determined that the struct to be written to file is
identical to data already in the file at that interval, nothing
is written and a 3 is returned.

*/

int putCrisSummary(struct L1CrisSummary *pDataStruct) {
    unsigned int DATA_TYPE_ID = FILE_CRIS_L1_SUMMARY;
    char FILE_TYPE_SUFFIX[] = "sum";

	/* declared static so memory allocation occurs only once */
    static char dataBuffer[sizeof(struct L1CrisSummary)];

    int fd, intervalNumber, fileInitialized, status, timeInterval;
    char path[256];
    struct tm *pTime_str;

	/* in the case of putCrisSummary, the returned interval is wrong */
    determineDataFilePathAndCreateDirsAsNeeded(pDataStruct->Second1996,
				FILE_TYPE_SUFFIX, path, &intervalNumber, 'C');

	/* putCrisSummary uses 1 second intervals instead of 256 seconds */
    timeInterval = 1;
    pTime_str = getDateFromSecondsSince1996(pDataStruct->Second1996);
    intervalNumber = pTime_str->tm_hour * 3600 +
				pTime_str->tm_min * 60 + pTime_str->tm_sec;

    if ((fd = open(path, O_RDWR)) == -1) {
	fileInitialized = 0;
	fd = openOrCreateFileToReadWrite(path);
    }
    else fileInitialized = 1;

    if (!fileInitialized) createInitializedFileWithIntervalN(fd,
		dataBuffer, sizeof(dataBuffer), DATA_TYPE_ID, timeInterval);

    status = writeDataStructToFile(fd, intervalNumber, dataBuffer,
				sizeof(dataBuffer), (char *) pDataStruct, path);
    close(fd);
    return status;
}
