#include <stdio.h>
#include <time.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "hdfi.h"
#include "utilities.h"
#include "crisSisPutUtilities.h"

/*

writeDataStructToFile takes a file descriptor, an interval number (where
in the set of 256 second blocks does the data belong?), an uninitialized
data buffer which is exactly the right size for one record of the data
in question, a pointer to the data structure to be written to file and
the path string which is used in case we need to create or write to an
auxiliary file.  writeDataStructToFile checks what has been written into
the data file for the interval where the passed data struct is to be written.

if the interval is all zeros (done at file initialization), the struct
is written there and a 0 is returned.

if the interval is found to contain data already, both the passed struct
and the one already in the file are written out to an auxiliary file
(same file name with ".overlap" suffix appended), the interval is filled
with char -1s, and a 1 is returned.

if the interval is filled with char -1s, then the passed struct is written
to the auxiliary file and a 2 is returned.

finally, if the passed data struct is identical to the data already in
the interval, nothing is written, and a 3 is returned.

*/

    /* remember, the dataBuffer is the same size as the data struct */
int writeDataStructToFile(int fd, int intervalNumber, char dataBuffer[],
				int bufferSize, char *pDataStruct, char *path) {
    unsigned long seekLocation;
    char contents;
    int i, auxFd;
    char auxPath[256];

    seekLocation = (unsigned long) HEADER_BYTES +
	((unsigned long) intervalNumber) * bufferSize;
    lseek(fd, seekLocation, 0);
    readFromFile(fd, dataBuffer, bufferSize);
	/* check the contents of this interval (slower but very general) */
    contents = dataBuffer[0];
    if ((contents != 0) && (contents != -1)) contents = 1;
    else {
	for (i = 1; i < bufferSize; i++) {
	    if (dataBuffer[i] != contents) {
		contents = 1;	/* signals data is already in this location */
		break;
	    }
	}
    }

    if (contents == 0) {
	    /* normal.  nothing has been written into this block yet. */
	lseek(fd, seekLocation, 0);
	writeToFile(fd, pDataStruct, bufferSize);
	return 0;	/* signal all is well */
    }

    else if (contents == 1) {
	/* bad thing.  some struct has already claimed this spot. */

	int dataIsIdentical;

	    /* find out if it is just an attempt to rewrite same data */
	dataIsIdentical = 1;
	for (i = 0; i < bufferSize; i++) {
	    if (*(pDataStruct + i) != dataBuffer[i]) {
		dataIsIdentical = 0;
		break;
	    }
	}
	if (dataIsIdentical) return 3;	/* do nothing */

	    /* write both of these structs to an auxiliary .overlap file. */
	sprintf(auxPath, "%s.overlap", path);
	auxFd = openOrCreateFileToReadWrite(auxPath);

	lseek(auxFd, 0L, 2);	/* go to end to append */
	    /* write the struct from the file to the aux file */
	writeToFile(auxFd, dataBuffer, bufferSize);
	    /* write the struct handed to this routine to the aux file */
	writeToFile(auxFd, pDataStruct, bufferSize);
	close(auxFd);
	    /* now fill this slot with char -1s to signal that overlapping */
	    /* files have been obtained for this interval */
	lseek(fd, seekLocation, 0);
	for (i = 0; i < bufferSize; i++) dataBuffer[i] = -1;
	writeToFile(fd, dataBuffer, bufferSize);
	return 1;	/* signal something is strange */
    }

    else if (contents == -1) {
	    /* very bad thing.  this is at least the third structure */
	    /* trying to claim this interval.  the others have already */
	    /* been written into the .overlap file.  append this one */
	    /* onto the .overlap file as well. */
	sprintf(auxPath, "%s.overlap", path);
	auxFd = openOrCreateFileToReadWrite(auxPath);
	lseek(auxFd, 0L, 2);      /* go to end to append */
	writeToFile(auxFd, pDataStruct, bufferSize);
	close(auxFd);
	return 2;	/* signal something is very strange */
    }
    return 4;		/* should never get here */
}
